package fr.unistra.sil.erp.back.repository.category;

import fr.unistra.sil.erp.back.model.Category;
import fr.unistra.sil.erp.back.repository.*;
import org.springframework.stereotype.Repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * SQLite Category SQL queries.
 * @author BEAUVAIS ANTOINE
 */
@Repository
public class SqliteCategoryRepository extends SqliteRepository implements ICategoriesRepository {

    public static String columns() {
        List<String> columns = List.of(
                "categories.id AS category_id",
                "categories.name AS category_name"
        );
        return String.join(", ", columns);
    }

    public static Category fromDb(ResultSet rs) throws SQLException {
        return new Category(rs.getInt("category_id"), rs.getString("category_name"));
    }

    /**
     * Query used to retrieve all categories stored in the database.
     */
    private static final String SQL_GETCATEGORIES =
            "SELECT " + columns() + " FROM categories";

    private static final String SQL_GET_CATEGORY = SQL_GETCATEGORIES + " WHERE id = ?";

    /**
     * Query used to create a new category.
     */
    private static final String SQL_CREATECATEGORY =
            "INSERT INTO categories (name) VALUES (?)";

    private static final String SQL_GETLASTCATEGORY =
            "SELECT MAX(id) as max_id, name FROM categories";

    /**
     *
     * @param conn
     */
    public SqliteCategoryRepository(Connection conn) {
        super(conn);
    }

    @Override
    public List<Category> getCategories() {
        ResultSet rs = this.query(SQL_GETCATEGORIES);
        if (rs == null)
            return null;

        List<Category> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Category c = fromDb(rs);
                res.add(c);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return res;
    }
    
    /**
     *
     * @param id
     * @return
     * @throws DatabaseResourceNotFoundException
     */
    public Category getCategory(int id) throws DatabaseResourceNotFoundException
    {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_GET_CATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, id);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch stock for item " +
                            id, ex);
            return null;
        }

        Category c;
        try {
            if (rs.next())
                c = fromDb(rs);
            else
                throw new DatabaseResourceNotFoundException("Stock not found.");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return c;
    }

    @Override
    public Category createCategory(Category c)
            throws DatabaseConnectionException, DatabaseUpdateException
    {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_CREATECATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            throw new DatabaseConnectionException("Failed to connect to DB.");
        }

        try {
            ps.setString(1, c.getName());
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            throw new DatabaseUpdateException("Failed to set query params.");
        }

        try {
            ps.execute();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName())
                    .log(Level.SEVERE, "Failed to execute query.", ex);
            throw new DatabaseUpdateException("Failed to execute query.");
        }

        int id;
        ResultSet rs;
        try {
            ps = this.conn.prepareStatement(SQL_GETLASTCATEGORY);
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to find item id: " + ex.getMessage());
            return null;
        }
        try {
            id = rs.getInt("max_id");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to parse item id: " + ex.getMessage());
            return null;
        }

        return new Category(id, c.getName());
    }
}
