package fr.unistra.sil.erp.back.repository.item;

import fr.unistra.sil.erp.back.model.Item;
import fr.unistra.sil.erp.back.repository.IItemsRepository;
import fr.unistra.sil.erp.back.repository.SqliteRepository;
import fr.unistra.sil.erp.back.repository.category.SqliteCategoryRepository;
import org.springframework.stereotype.Repository;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author antoine.beauvais
 */
@Repository
public class SqliteItemsRepository extends SqliteRepository implements IItemsRepository {

    public static String columns() {
        List<String> columns = List.of(
                "items.id AS item_id",
                "items.name AS item_name",
                "items.price AS item_price",
                "items.subscriberPrice AS item_subscriber_price",
                "items.category AS item_category"
        );
        return String.join(", ", columns) + ", " + SqliteCategoryRepository.columns();
    }

    public static Item fromDb(ResultSet rs) throws SQLException {
        return new Item(rs.getInt("item_id"),
                rs.getString("item_name"),
                rs.getBigDecimal("item_price"),
                rs.getBigDecimal("item_subscriber_price"),
                SqliteCategoryRepository.fromDb(rs)
        );
    }

    /**
     * Query used to find an item by id
     */
    private static final String SQL_GET_ITEM = "SELECT " + columns() + " FROM items WHERE id = ?";

    private static final String SQL_GETITEMSWITHCATEGORY =
            "SELECT " + columns() + " FROM items INNER JOIN categories ON items.category = categories.id";

    /**
     * Query used to get all items stored in the database from a specific
     * category.
     */
    private static final String SQL_GETITEMSFROMCATEGORY = SQL_GETITEMSWITHCATEGORY +
            " WHERE categories.id = ?";


    /**
     * Class constructor.
     *
     * @param conn
     */
    public SqliteItemsRepository(Connection conn) {
        super(conn);
    }


    @Override
    public List<Item> getAllItems() {
        ResultSet rs = query(SQL_GETITEMSWITHCATEGORY);
        if (rs == null)
            return null;

        List<Item> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Item item = fromDb(rs);
                res.add(item);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, null, ex);
            return null;
        }

        return res;
    }

    @Override
    public List<Item> getAllItemsWithCategory() {
        ResultSet rs = query(SQL_GETITEMSWITHCATEGORY);
        Logger.getLogger(this.getClass().getName()).log(
                Level.SEVERE, SQL_GETITEMSWITHCATEGORY);
        if (rs == null)
            return null;

        List<Item> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Item item = fromDb(rs);
                res.add(item);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, null, ex);
            return null;
        }

        return res;
    }


    @Override
    public List<Item> getItemsFromCategory(int category) {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(
                    SQL_GETITEMSFROMCATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, category);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch items for category " +
                            category, ex);
            return null;
        }

        List<Item> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Item item = fromDb(rs);
                res.add(item);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return res;
    }

    /**
     * @param item
     * @return
     */
    @Override
    public Item store(Item item) {
        String query = "INSERT INTO items (name, price, subscriberPrice, category) VALUES (?, ?, ?, ?);";
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(query);
            ps.setString(1, item.getName());
            ps.setBigDecimal(2, item.getPrice());
            ps.setBigDecimal(3, item.getSubscriberPrice());
            ps.setInt(4, item.getCategory().getId());
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }
        try {
            ps.executeUpdate();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to create item: " + ex.getMessage());
            return null;
        }
        int id;
        query = "SELECT MAX(id) AS max_id FROM items";
        ResultSet rs;
        try {
            ps = this.conn.prepareStatement(query);
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to find item id: " + ex.getMessage());
            return null;
        }
        try {
            id = rs.getInt("max_id");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to parse item id: " + ex.getMessage());
            return null;
        }
        return new Item(
                id,
                item.getName(),
                item.getPrice(),
                item.getSubscriberPrice(),
                item.getCategory()
        );
    }

    /**
     * @param id
     * @return
     */
    @Override
    public Item findById(int id) {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_GET_ITEM);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, id);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to fetch item");
            return null;
        }
        Item item;
        try {
            if (rs.next())
                item = fromDb(rs);
            else
                return null;
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return item;
    }
}
